// common module used in both, server and client programs
unit _12Common;

interface

uses Classes, ScktComp;

type
  TRangeKind = (rkPending, rkDone, rkAborted);
  // kind of range
  // rkPending -- calculations are beeing done on client
  // rkDone -- calculations are already done
  // rkAborted -- connection was lost to client during calculations

  TRange = class // range of numbers which are send to server
    intBegin: Integer;  // begin number
    intEnd: Integer;    // end number
    Kind: TRangeKind;   // range kind (stored on server)
    Numbers: TList;     // founded prime numbers
  end;

  TJSocket = class
    Socket: TCustomWinSocket;  // socket
    Text: String;   // text which was reveived from socket
  end;
  
const  // commands between server and client
  cmdEndCommand = #13;

  cmdAddRange = #56;
  cmdRequestRange = #57;
  cmdRequestNumbers = #58;
  cmdFoundNumbers = #59;

  cmdNewRange = #60;
  cmdNumbers = #61;
  

procedure DeleteRange(ARanges: TList; ID: Integer);
function GetNewRange(ARanges: TList; AMin, AMax: Integer): TRange;
procedure OptimizeRanges(ARanges: TList);
function GetParam(S: String; ID: Integer): String;

implementation

uses Math;


function GetParam(S: String; ID: Integer): String;
var
  p, i: Integer;
begin
  Result := '';
  i := 0;
  while Pos(' ', S) <> 0 do
  begin
    p := Pos(' ', S);
    S := Copy(S, p + 1, MaxInt);
    Inc(i);
    if i = ID then
    begin
      p := Pos(' ', S);
      if p = 0 then
        p := Length(S) + 1;
      Result := Copy(S, 1, p - 1);
    end;
  end;
end;

procedure DeleteRange(ARanges: TList; ID: Integer);
begin
  TRange(ARanges[ID]).Numbers.Free;
  TRange(ARanges[ID]).Free;
  ARanges.Delete(ID);
end;

procedure OptimizeRanges(ARanges: TList);
var
  a, b, c: Integer;
  f: Boolean;
  r1, r2: TRange;
begin
  a := 0;
  f := True;
  while (a <= ARanges.Count - 1) and f do
  begin
    if TRange(ARanges[a]).Kind = rkAborted then
    begin
      DeleteRange(ARanges, a);
      OptimizeRanges(ARanges);
      f := False;
    end else
      if TRange(ARanges[a]).Kind = rkDone then
      begin
        b := 0;
        while (b <= ARanges.Count - 1) and f do
        begin
          if (b <> a) and (TRange(ARanges[b]).Kind = rkDone) then
          begin
            r1 := TRange(ARanges[a]);
            r2 := TRange(ARanges[b]);
            if r2.intBegin - 1 = r1.intEnd then
            begin
              r1.intEnd := r2.intEnd;
              for c := 0 to r2.Numbers.Count - 1 do
                r1.Numbers.Add(r2.Numbers[c]);
              DeleteRange(ARanges, b);
              f := False;
            end;
          end;
          Inc(b);
        end;
      end;

    Inc(a);
  end;
end;

function GetNewRange(ARanges: TList; AMin, AMax: Integer): TRange;
var
  a, b, e: Integer;
  r: TRange;
begin
  OptimizeRanges(ARanges);

  b := -1;
  for a := 0 to ARanges.Count - 1 do
  begin
    r := TRange(ARanges[a]);
    if r.intEnd + 1> b then
      b := r.intEnd + 1;
  end;
  if b = -1 then
    b := AMin;

  e := Min(b + 9999, AMax);

  for a := 0 to ARanges.Count - 1 do
  begin
    r := TRange(ARanges[a]);
    if r.intBegin - 1 < e then
      e := r.intBegin - 1;
  end;
  
  Result := TRange.Create;
  Result.intBegin := b;
  Result.intEnd := e;
  Result.Kind := rkPending;
  ARanges.Add(Result);
 
end;

end.
